// File created 03/12/2015 with four space tab spacing

/** \file
	\brief Clock timer management
*/

#include "clock_timer.h"
#include "simpletools.h"
#include "config.h"
#include "dda.h"

/// every time our clock fires, we increment this so we know when 10ms has elapsed
uint8_t clock_counter_10ms = 0;

/// keep track of when 250ms has elapsed
uint8_t clock_counter_250ms = 0;

/// keep track of when 1s has elapsed
uint8_t clock_counter_1s = 0;

/// flags to tell main loop when above have elapsed
volatile uint8_t clock_flag_10ms = 0;
volatile uint8_t clock_flag_250ms = 0;
volatile uint8_t clock_flag_1s = 0;

/// cog variables
static int cog = 0;
static int stack[60];

void clock_timer(void *par);

/// initialise timer
void clock_timer_init()
{
	if(cog == 0)
	{
		cog = 1 + cogstart(&clock_timer, NULL, stack, sizeof stack);
	}
}

/// stop timers
void clock_timer_stop()
{
	if(cog > 0)
	{
		cogstop(cog - 1);
		cog = 0;
	}
}

/// maintain state of the clock flags
void clock_timer(void *par)
{
	while(1)
	{
		waitcnt(TICK_TIME + CNT);

		clock_counter_10ms += TICK_TIME;

		if(clock_counter_10ms >= 10)
		{
			clock_counter_10ms -= 10;
			clock_flag_10ms = 1;

			clock_counter_250ms++;

			if(clock_counter_250ms >= 25)
			{
				clock_counter_250ms = 0;
				clock_flag_250ms = 1;

				clock_counter_1s++;

				if(clock_counter_1s >= 4)
				{
					clock_counter_1s = 0;
					clock_flag_1s = 1;
				}
			}
		}

		dda_clock();
	}                            
}